use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsClassByAlias
  LoadClassByAlias
  LoadRootObject
);
use DE_EPAGES::RemoteSearch::API::AttributeModifiedCheck qw ( RunIfModified );
use Test::More;

my $ROOTALIAS = 'Test';
my $TESTCLASS = 'TestRemoteSearchObjectClass';

sub init {
    return 1 if LoadRootObject->existsChild($ROOTALIAS) && $ENV{'SKIP_CLEANUP'};
    cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    my $ClassObject = LoadClassByAlias('Object');
    my $Class1 = $ClassObject->insertSubClass( { Alias => $TESTCLASS } );
    $Class1->insertAttribute( { Alias => 'Name',    Type => 'LocalizedString', Package => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute' } );
    $Class1->insertAttribute( { Alias => 'MyFloat', Type => 'Float',           Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute' } );
    return 1;
}

sub cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    if ( ExistsClassByAlias($TESTCLASS) ) {
        LoadClassByAlias($TESTCLASS)->delete;
    }
    LoadRootObject->child($ROOTALIAS)->delete if LoadRootObject->existsChild($ROOTALIAS);
}

sub testRunIfModified {
    my $Object      = LoadRootObject()->child($ROOTALIAS)->insertChild( $TESTCLASS, { Alias => 'ModifiedTestObject' } );
    my $aAttributes = [qw ( Alias Position )];
    my $LanguageID  = undef;
    my $aTypes      = [qw( String Integer )];
    my $hNewValues  = { Alias => 'ModifiedTestObject', 'Position' => 9999 };
    my $a           = 0;
    my $b           = 0;
    my $cSub        = sub { ++$a };
    RunIfModified( $Object, $aAttributes, $hNewValues, $LanguageID, $aTypes, $cSub );
    is( $a, ++$b, 'csub executed' );

    $hNewValues->{'Position'} = $Object->get('Position');
    RunIfModified( $Object, $aAttributes, $hNewValues, $LanguageID, $aTypes, $cSub );
    is( $a, $b, 'csub not executed' );

    $aAttributes = ['Name'];
    $aTypes      = ['String'];
    $LanguageID  = 1;

    RunIfModified( $Object, $aAttributes, $hNewValues, $LanguageID, $aTypes, $cSub );
    is( $a, $b, 'csub not executed (Name not in hNewValues)' );

    $hNewValues = { 'Name' => 'testing' };
    RunIfModified( $Object, $aAttributes, $hNewValues, $LanguageID, $aTypes, $cSub );
    is( $a, ++$b, 'csub executed (Name in hNewValues), Name undef in db.' );

    $Object->set( { 'Name' => 'testing' }, $LanguageID );
    RunIfModified( $Object, $aAttributes, $hNewValues, $LanguageID, $aTypes, $cSub );
    is( $a, $b, 'csub not executed (Name equals hNewValues), Name def in db.' );

    $hNewValues = { 'Name' => undef };
    RunIfModified( $Object, $aAttributes, $hNewValues, $LanguageID, $aTypes, $cSub );
    is( $a, ++$b, 'csub executed (Name undef in hNewValues), Name def in db.' );

    $hNewValues  = { 'MyFloat' => 1.1 };
    $aAttributes = [qw ( MyFloat )];
    $aTypes      = [qw ( Float )];

    RunIfModified( $Object, $aAttributes, $hNewValues, undef, $aTypes, $cSub );
    is( $a, ++$b, 'csub executed (Float check), MyFloat undef in db.' );

    $Object->set( { 'MyFloat' => 1.1 } );
    RunIfModified( $Object, $aAttributes, $hNewValues, undef, $aTypes, $cSub );
    is( $a, $b, 'csub not executed (Float check), MyFloat equals in db.' );

    $Object->set( { 'MyFloat' => 1.2 } );
    RunIfModified( $Object, $aAttributes, $hNewValues, undef, $aTypes, $cSub );
    is( $a, ++$b, 'csub executed (Float check), MyFloat not equals in db.' );

    $hNewValues->{'MyFloat'} = 1.3;
    RunIfModified( sub { return {'MyFloat' => 1.3 }}, $aAttributes, $hNewValues, undef, $aTypes, $cSub );
    is( $a, $b, 'csub not executed (using coderef for checking), MyFloat equals' );

    $hNewValues->{'MyFloat'} = 1.4;
    RunIfModified( sub { return {'MyFloat' => 1.3 }}, $aAttributes, $hNewValues, undef, $aTypes, $cSub );
    is( $a, ++$b, 'csub not using coderef for checking, MyFloat non-equals' );
}

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # tests
            init();
            pass('init');
            testRunIfModified();
            pass('test execution');
            cleanup();
            pass('cleanup');
            done_testing();
        }
    );
}

RunScript( Sub => \&Main, Debug => 1 );

1;
